#version 150
precision highp float; precision highp int;
//Ethan Alexander Shulman 2023 "Wobble Bop" February 14, 2023

in vec3 in_Position;
out vec4 uv;
out float time;

layout(std140) uniform ShaderviewBlock {
	vec4 mouse;
	float screenX,screenY;
	int frame,framerate;
};
layout(std140) uniform MidiBlock {
	vec4 midi[64];//x = note id, y = velocity, z = channel/track, w = time in seconds since note pressed
	int midiCount;
};
out vec4 music[2];//0.x = kick, 0.y = stick, 0.z = hat, 0.w = organ, 1.x = bass wobble

/*LMMS midi converter https://www.lynxwave.com/LMMStoMIDI/LMMStoMIDIConverter.html since LMMS midi export is broken.

Launch options for midi config 'Shaderview console 21stCentury.mid 31 0.4'
31 is the bitmask 0b0011111 signifying which instruments are beats/drums with a fixed length.
0.4 is the fixed beat length in seconds.

Midi Instrument 0 = 512lotom.wav
Midi Instrument 1 = MaxV - MACH-KCK.wav
Midi Instrument 1 = 512hhcl.wav
Midi Instrument 2 = 512cymb.wav
Midi Instrument 3 = MaxV - DANCE-S2.wav
Midi Instrument 4 = TripleOscillator
Midi Instrument 5 = Organic*/

#define TEMPO_TIME(bpm,note) (60.*4.*note/bpm)


//#define ANIMATION 1
#define AA_SAMPLES 10000000


#define PI 3.141592653589793


mat2 r2d(float a) {
	float c = cos(a), s = sin(a);
	return mat2(c,s,-s,c);
}

//r2 hash http://extremelearning.com.au/unreasonable-effectiveness-of-quasirandom-sequences/
vec2 quasiRandom(float n) {
	#define R2V 1.32471795724474602596
	return fract(.5+n*(1./vec2(R2V,R2V*R2V)));
}


vec3 hue(float h) {
	return clamp(abs(mod(h*6.+vec3(0.,4.,2.),6.)-3.)-1.,0.,1.);
}



uniform sampler2D Buffer0;

void vert() {
	#ifdef ANIMATION
	time = float(frame)/float(framerate*AA_SAMPLES);
	music[0] = vec4(0);
	music[1] = vec4(0);
	for (int i = 0; i < midiCount; i++) {
		if (midi[i].z == 0.) {//kick
			music[0].x += max(0.,1.-midi[i].w/.35);
		} else if (midi[i].z == 1.) {//hat
			music[0].z += max(0.,1.-midi[i].w/.2);
		} else if (midi[i].z == 3.) {//stick
			music[0].y += max(0.,1.-midi[i].w/.2);
		} else if (midi[i].z == 4.) {//bass
			music[1].x++;
		} else if (midi[i].z == 5.) {//organ
			music[0].w++;
		}
	}
	
	#else
	time = 0.;
	music[0] = vec4(1,1,1,1);
	music[1] = vec4(.5);
	#endif
	
	uv = vec4(in_Position.xy*vec2(screenX/screenY,1)+(quasiRandom(float(frame%AA_SAMPLES))-.5)*2./screenY,
		(in_Position.xy*.5+.5)*vec2(screenX,screenY));
	gl_Position = vec4(in_Position,1);
}
void frag() {
	float camAngle = mod(floor(floor(time/TEMPO_TIME(140.,5.))*2.6881264),3.);
	vec2 camRot;
	vec3 camPos;
	if (camAngle == 0.) {
		camRot = vec2(.3,0);
		camPos = vec3(.5,.6,-1.2);
	} else if (camAngle == 1.) {
		camRot = vec2(0,-.7);
		camPos = vec3(0,1,-1);
	} else {
		camRot = vec2(0,-PI*.5);
		camPos = vec3(0,2,0);
	}
	
	vec3 rp = camPos,
		rd = normalize(vec3(uv.xy,1));
	rd.yz *= r2d(camRot.y);
	rd.xz *= r2d(camRot.x);
	
	vec3 c = vec3(0);
	for (float f = .5-fract(float(frame%AA_SAMPLES)*.778234)*0.05; f < 4.; f += .05) {
		vec3 p = rp+rd*f;
		float yd = 0., ay = abs(p.y);
		
		#define WV(fx,fz,am) yd += cos(dot(p.xz,vec2(fx,fz)))*max(0.,1.-abs(dot(p.xz,normalize(vec2(fz,-(fx))))))*(am)
		WV(6.,0., music[0].w*.5);//organ
		WV(2.,2., music[0].y*.4);//stick
		WV(-2.,2., music[0].z*.4);//hat
		WV(0.,3., music[0].x*-.75);//kick
		WV(4.,0., music[1].x*cos(time*PI*2./TEMPO_TIME(140.,1./8.)));//bass wobble
		
		c += hue(ay+.25)*clamp(ay-.25,0.,1.)*max(0., 1.-abs(p.y-yd)*4.);
	}
	c *= .4;
	
	vec4 sum = texelFetch(Buffer0,ivec2(uv.zw),0);
	if (frame%AA_SAMPLES == 0) sum = vec4(0);
	sum += vec4(c, 1.);
	gl_FragColor = sum;
}
